from pathlib import Path

from iccore.project import version, Version


_DEFAULT_CONF = Path("docs/conf.py")


def set_version(path: Path, ver: Version, conf_path: Path = _DEFAULT_CONF):
    conf_path = path / conf_path
    with open(conf_path, "r") as f:
        lines = f.readlines()

    with open(conf_path, "w") as f:
        for line in lines:
            if "release" in line:
                line = f"release = '{ver.as_string()}'"
            f.write(line)


def _parse_version(line: str) -> Version:
    _, val = line.split("=")
    return version.parse(val.strip().replace("'", ""))


def get_version(path: Path, conf_path: Path = _DEFAULT_CONF) -> Version:
    with open(path / conf_path, "r") as f:
        lines = f.readlines()

    for line in lines:
        if "release" in line:
            return _parse_version(line)
    raise RuntimeError("release key not found in config")



import os
import logging
from pathlib import Path

from .ci import gitlab_ci_push, sync_external_archive

logger = logging.getLogger(__name__)


def gitlab_ci_push_cli(args):

    logger.info("CI pushing state of current checkout")

    gitlab_ci_push(
        args.user_name, args.user_email, args.repo_url, args.token, args.message
    )

    logger.info("CI finished pushing state of current checkout")


def register_ci_push(subparsers):

    parser = subparsers.add_parser("ci_push")
    parser.add_argument("--user_name", type=str, help="Name of the CI user")
    parser.add_argument("--user_email", type=str, help="Email of the CI user")
    parser.add_argument(
        "--instance_url", type=str, help="Url for the target ci instance"
    )
    parser.add_argument(
        "--url", type=str, help="Url for the repo relative to the ci instance"
    )
    parser.add_argument("--token", type=str, help="Oath access token for the repo")
    parser.add_argument("--message", type=str, help="Commit message")
    parser.set_defaults(func=gitlab_ci_push_cli)


def sync_external_archive_cli(args):

    logger.info("Starting external package sync")

    sync_external_archive(
        repo_dir=args.repo_dir,
        project_id=args.project_id,
        sync_script=args.sync_script,
        asset_name=args.asset_name,
        archive_name=args.archive_name,
        source_token=args.source_token,
        user_name=args.user_name,
        user_email=args.user_email,
        url=args.url,
        target_token=args.target_token,
        target_repo_url=args.target_repo_url,
    )

    logger.info("Finished external package sync")


def register_sync_exterinal(subparsers):

    parser = subparsers.add_parser("sync_external_archive")
    parser.add_argument(
        "--repo_dir",
        type=Path,
        default=Path(os.getcwd()),
        help="Path to the repo to increment the tag",
    )

    parser.add_argument(
        "--project_id",
        type=int,
        help="The id of the project to sync against.",
    )
    parser.add_argument(
        "--source_token", type=str, default="", help="Access token for the source repo"
    )

    parser.add_argument("--user_name", type=str, default="", help="Name of the CI user")
    parser.add_argument(
        "--user_email", type=str, default="", help="Email of the CI user"
    )

    parser.add_argument(
        "--url",
        type=str,
        default="https://git.ichec.ie",
        help="Url for the Gitlab instance",
    )
    parser.add_argument(
        "--target_token", type=str, default="", help="Oath access token for the repo"
    )

    parser.add_argument(
        "--asset_name", type=str, help="Name of the release asset for the archive"
    )
    parser.add_argument(
        "--archive_name", type=str, help="Name of the downloaded archive"
    )
    parser.add_argument(
        "--sync_script", type=Path, help="Path to script to do the sync"
    )
    parser.add_argument(
        "--target_repo_url", type=str, default="", help="Url for the target repo"
    )
    parser.set_defaults(func=sync_external_archive_cli)


def register(subparser):

    register_ci_push(subparser)
    register_sync_exterinal(subparser)



from pathlib import Path
import logging

from pydantic import BaseModel

from iccore.system import process


logger = logging.getLogger(__name__)


class PyPiContext(BaseModel, frozen=True):

    token: str
    use_test_repo: bool = False


class PythonPackage(BaseModel, frozen=True):
    repo_path: Path


def build(package: PythonPackage):
    logger.info("Building Python package")
    output_dir = package.repo_path / "dist"
    cmd = f"python3 -m build --outdir {output_dir} {package.repo_path}"
    process.run(cmd)
    logger.info("Finished building Python package")


def upload(package: PythonPackage, pypi_context: PyPiContext):
    logger.info("Uploading Python package")

    if not pypi_context.token:
        raise RuntimeError("Provided PyPi token cannot be empty")

    repo = ""
    if pypi_context.use_test_repo:
        repo = " -r testpypi "

    token = f"-p {pypi_context.token}"
    upload_dir = package.repo_path / "dist"
    cmd = f"twine upload {upload_dir}/*  --non-interactive {token} {repo}"
    process.run(cmd)
    logger.info("Finished uploading Python package")


def deploy(repo_path: Path, token: str, use_test_repo: bool = False):
    package = PythonPackage(repo_path=repo_path)
    build(package)
    upload(package, PyPiContext(token=token, use_test_repo=use_test_repo))



from pathlib import Path
import shutil
import os

from iccore.test_utils import get_test_data_dir, get_test_output_dir

from iccicd.repo import python_repo, Repo


def setup_mock_repo(name, output_dir):

    repo_dir = get_test_data_dir() / name
    output_repo_dir = output_dir / name
    os.makedirs(output_dir)
    shutil.copytree(repo_dir, output_repo_dir)
    shutil.move(
        output_repo_dir / "testpyproject.toml", output_repo_dir / "pyproject.toml"
    )
    shutil.move(output_repo_dir / "docs/conf.xpy", output_repo_dir / "docs/conf.py")


def test_increment_python_repo_version():

    name = "version_bump"
    output_dir = get_test_output_dir()
    setup_mock_repo(name, output_dir)

    version = python_repo.get_version(output_dir / name)

    python_repo.increment_version(
        repo=Repo(path=output_dir / name, version=version), field="minor"
    )

    pp_version = python_repo.pyproject.get_version(output_dir / name)
    sphinx_version = python_repo.sphinx.get_version(output_dir / name)

    shutil.rmtree(output_dir)

    assert pp_version.major == 0
    assert pp_version.minor == 1
    assert pp_version.patch == 0

    assert sphinx_version.major == 0
    assert sphinx_version.minor == 1
    assert sphinx_version.patch == 0



from pathlib import Path
from typing import Any, cast
import logging
import tomlkit
from tomlkit import TOMLDocument

from iccore.project import version, Version

logger = logging.getLogger(__name__)


_DEFAULT_FILENAME = "pyproject.toml"


def read(path: Path, filename: str = _DEFAULT_FILENAME) -> dict[str, Any]:
    with open(path / filename, "r") as f:
        content = f.read()
    return tomlkit.parse(content)


def write(path: Path, content: dict[str, Any], filename: str = _DEFAULT_FILENAME):
    with open(path / filename, "w") as f:
        f.write(tomlkit.dumps(cast("TOMLDocument", content)))


def get_version(path: Path, filename: str = _DEFAULT_FILENAME) -> Version:
    content = read(path, filename)
    return version.parse(content["project"]["version"])


def set_version(path: Path, ver: Version, filename: str = _DEFAULT_FILENAME):
    content = read(path, filename)
    content["project"]["version"] = ver.as_string()
    write(path, content, filename)
